/*
 * $Id$
 *
 * Copyright (C) 2003 ETC s.r.o.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Written by Marcel Telka <marcel@telka.sk>, 2003.
 *
 */


/*
 * Please keep this list sorted alphabetically
 */

#ifdef ENABLE_BUS_AHBJTAG
_URJ_BUS(ahbjtag)
#endif
#ifdef ENABLE_BUS_ARM9TDMI
_URJ_BUS(arm9tdmi)
#endif
#ifdef ENABLE_BUS_AU1500
_URJ_BUS(au1500)
#endif
#ifdef ENABLE_BUS_AVR32
_URJ_BUS(avr32)
#endif
#ifdef ENABLE_BUS_BCM1250
_URJ_BUS(bcm1250)
#endif
#ifdef ENABLE_BUS_BLACKFIN
_URJ_BUS(bf518f_ezbrd)
_URJ_BUS(bf51x)
_URJ_BUS(bf526_ezkit)
_URJ_BUS(bf527_ezkit)
_URJ_BUS(bf527_sdp)
_URJ_BUS(bf52x)
_URJ_BUS(bf533_ezkit)
_URJ_BUS(bf533_stamp)
_URJ_BUS(bf537_ezkit)
_URJ_BUS(bf537_stamp)
_URJ_BUS(bf538f_ezkit)
_URJ_BUS(bf53x)
_URJ_BUS(bf548_ezkit)
_URJ_BUS(bf561_ezkit)
#endif
#ifdef ENABLE_BUS_BSCOACH
_URJ_BUS(bscoach)
#endif
#ifdef ENABLE_BUS_EJTAG
_URJ_BUS(ejtag)
_URJ_BUS(ejtag_dma)
#endif
#ifdef ENABLE_BUS_FJMEM
_URJ_BUS(fjmem)
#endif
#ifdef ENABLE_BUS_H7202
_URJ_BUS(h7202)
#endif
#ifdef ENABLE_BUS_IXP425
_URJ_BUS(ixp425)
#endif
#ifdef ENABLE_BUS_IXP435
_URJ_BUS(ixp435)
#endif
#ifdef ENABLE_BUS_IXP465
_URJ_BUS(ixp465)
#endif
#ifdef ENABLE_BUS_JOPCYC
_URJ_BUS(jopcyc)
#endif
#ifdef ENABLE_BUS_LH7A400
_URJ_BUS(lh7a400)
#endif
#ifdef ENABLE_BUS_MPC5200
_URJ_BUS(mpc5200)
#endif
#ifdef ENABLE_BUS_MPC824X
_URJ_BUS(mpc824x)
#endif
#ifdef ENABLE_BUS_MPC8313
_URJ_BUS(mpc8313)
#endif
#ifdef ENABLE_BUS_MPC837X
_URJ_BUS(mpc837x)
#endif
#ifdef ENABLE_BUS_PPC405EP
_URJ_BUS(ppc405ep)
#endif
#ifdef ENABLE_BUS_PPC440GX_EBC8
_URJ_BUS(ppc440gx_ebc8)
#endif
#ifdef ENABLE_BUS_PROTOTYPE
_URJ_BUS(prototype)
#endif
#ifdef ENABLE_BUS_PXA27X
_URJ_BUS(pxa27x)
#endif
#ifdef ENABLE_BUS_PXA2X0
_URJ_BUS(pxa2x0)
#endif
#ifdef ENABLE_BUS_S3C4510
_URJ_BUS(s3c4510)
#endif
#ifdef ENABLE_BUS_SA1110
_URJ_BUS(sa1110)
#endif
#ifdef ENABLE_BUS_SH7727
_URJ_BUS(sh7727)
#endif
#ifdef ENABLE_BUS_SH7750R
_URJ_BUS(sh7750r)
#endif
#ifdef ENABLE_BUS_SH7751R
_URJ_BUS(sh7751r)
#endif
#ifdef ENABLE_BUS_SHARC_21065L
_URJ_BUS(sharc_21065L)
#endif
#ifdef ENABLE_BUS_SHARC_21369_EZKIT
_URJ_BUS(sharc_21369_ezkit)
#endif
#ifdef ENABLE_BUS_SLSUP3
_URJ_BUS(slsup3)
#endif
#ifdef ENABLE_BUS_TX4925
_URJ_BUS(tx4925)
#endif
#ifdef ENABLE_BUS_ZEFANT_XS3
_URJ_BUS(zefant_xs3)
#endif

#undef _URJ_BUS
